@@ -256,9 +256,11 @@ pub enum Descriptor {
 pub struct RemoteNodeMutable {
 	state: State,
 	// last sent message number
-	message_sequence_number: u64,
+	last_sent_message_sequence_number: u64,
 	// last received message number
-	message_acknowledgment_number: u64,
+	last_received_message_sequence_number: u64,
+	// last sent acknowledgment
+	last_sent_message_acknowledgement_number: u64,
 	diff_produce: Option<bool>,
 	diff_request: Option<bool>,
 }
@@ -275,6 +277,7 @@ pub struct RemoteNode {
 
 impl Drop for RemoteNode {
 	fn drop(&mut self) {
+		trace!("Dropping RemoteNode {:?}", self.node_core);
 		//FORMER-BUG, objects and tags clear was missing, leading to wrong ref counts
 		//REGRESSION-TEST-NEEDED
 		let mut mutable = self.mutable.lock().unwrap();
@@ -309,7 +312,14 @@ impl RemoteNode {
 			node_core: NodeCore::new(upstream, Some(0), Some(0), None),
 			local_node: local_node.shared.clone(),
 			local_node_events,
-			mutable: Mutex::new(RemoteNodeMutable { state, message_sequence_number: 0, message_acknowledgment_number: 0, diff_produce, diff_request }),
+			mutable: Mutex::new(RemoteNodeMutable {
+				state,
+				last_sent_message_sequence_number: 0,
+				last_received_message_sequence_number: 0,
+				last_sent_message_acknowledgement_number: 0,
+				diff_produce,
+				diff_request,
+			}),
 		}
 	}
 
@@ -341,8 +351,23 @@ impl RemoteNode {
 		mutable.state.objects.clear();
 		mutable.state.tags.clear();
 		mutable.state.options.diff_request = None;
-		mutable.message_sequence_number = 0;
-		mutable.message_acknowledgment_number = 0;
+		mutable.last_sent_message_sequence_number = 0;
+		mutable.last_received_message_sequence_number = 0;
+		mutable.last_sent_message_acknowledgement_number = 0;
+	}
+
+	pub fn ack(&self) -> Option<Message> {
+		let mut mutable = self.mutable.lock().unwrap();
+		if mutable.last_sent_message_acknowledgement_number == mutable.last_received_message_sequence_number {
+			None
+		} else {
+			mutable.last_sent_message_acknowledgement_number = mutable.last_received_message_sequence_number;
+			Some(Message {
+				sequence_number: mutable.last_sent_message_sequence_number,
+				acknowledgment_number: mutable.last_received_message_sequence_number,
+				changes: vec![],
+			})
+		}
 	}
 
 	//TODO replace String error with proper error enum
@@ -351,7 +376,7 @@ impl RemoteNode {
 
 		let changes_to_send = vec![];
 		let mut mutable = self.mutable.lock().unwrap();
-		mutable.message_acknowledgment_number = message.sequence_number;
+		mutable.last_received_message_sequence_number = message.sequence_number;
 
 		for change in message.changes {
 			match change {
@@ -501,8 +526,13 @@ impl RemoteNode {
 		if changes.is_empty() {
 			vec![]
 		} else {
-			mutable.message_sequence_number += 1;
-			vec![Message { sequence_number: mutable.message_sequence_number, acknowledgment_number: mutable.message_acknowledgment_number, changes }]
+			mutable.last_sent_message_sequence_number += 1;
+			mutable.last_sent_message_acknowledgement_number = mutable.last_received_message_sequence_number;
+			vec![Message {
+				sequence_number: mutable.last_sent_message_sequence_number,
+				acknowledgment_number: mutable.last_received_message_sequence_number,
+				changes,
+			}]
 		}
 	}
 
@@ -613,7 +643,12 @@ impl RemoteNode {
 
 					if !change.is_empty() {
 						if change.data_version.is_some() {
-							object_state.sent.data.push_back((mutable.message_sequence_number, data_version.unwrap(), change.data.is_some(), data.unwrap()));
+							object_state.sent.data.push_back((
+								mutable.last_sent_message_sequence_number,
+								data_version.unwrap(),
+								change.data.is_some(),
+								data.unwrap(),
+							));
 						}
 						changes.push(Change::Object(change));
 					};
@@ -699,10 +734,12 @@ impl RemoteNode {
 		for (descriptor, state) in &mutable.state.objects {
 			ret.push_str(
 				format!(
-					"│ {:?} Received({}) Sent({})\n",
+					"│ {:?} Received({}) Sent({}) ReceivedVersionsStored:{}  SentVersionsStored:{}\n",
 					&descriptor,
 					vec![if state.received.observed { "observed" } else { "" }, if state.received.exposed { "exposed" } else { "" }].join(","),
 					vec![if state.sent.observed { "observed" } else { "" }, if state.sent.exposed { "exposed" } else { "" }].join(","),
+					state.received.data.len(),
+					state.sent.data.len()
 				)
 				.as_str(),
 			);
