@@ -1,7 +1,7 @@
 /*! Provides Websocket transport running in Tokio runtime
 */
 
-use std::{collections::HashMap, convert::TryInto, sync::{Arc, Mutex, RwLock}, time::{Duration, Instant}};
+use std::{cmp::min, collections::HashMap, convert::TryInto, sync::{Arc, Mutex, RwLock}, time::{Duration, Instant}};
 
 use async_tungstenite::{tokio::connect_async, tungstenite};
 use futures::{stream::{SplitSink, SplitStream}, SinkExt, StreamExt};
@@ -567,23 +567,34 @@ where
 		Ok(())
 	}
 
+	//TODO: move keep-alives and acks to remote_node_event_loop()
 	async fn receive(&self) -> Result<Message, Termination> {
 		let mut ws_stream_read = self.ws_stream_read.lock().await;
 		loop {
-			// sending keep-alives if needed
+			// sending ack if needed
+			if Instant::now() > { *self.last_sent_at.lock().unwrap() } + Duration::from_secs(1) {
+				if let Some(message) = self.remote_node.ack() {
+					self.send(vec![message]).await?;
+				}
+			}
+			// sending ack or keep-alive if needed
 			if Instant::now() > { *self.last_sent_at.lock().unwrap() } + self.parameters.timeout / 2 {
-				*self.last_sent_at.lock().unwrap() = Instant::now();
-				self.ws_stream_write
-					.lock()
-					.await
-					.as_mut()
-					.unwrap()
-					.send(tungstenite::Message::Ping(vec![]))
-					.await
-					.map_err(Termination::from_tungstenite_error)?;
+				if let Some(message) = self.remote_node.ack() {
+					self.send(vec![message]).await?;
+				} else {
+					*self.last_sent_at.lock().unwrap() = Instant::now();
+					self.ws_stream_write
+						.lock()
+						.await
+						.as_mut()
+						.unwrap()
+						.send(tungstenite::Message::Ping(vec![]))
+						.await
+						.map_err(Termination::from_tungstenite_error)?;
+				}
 			};
 			// trying to receive something
-			if let Ok(message) = timeout(self.parameters.timeout / 4, ws_stream_read.as_mut().unwrap().next()).await {
+			if let Ok(message) = timeout(min(self.parameters.timeout / 4, Duration::from_micros(1000)), ws_stream_read.as_mut().unwrap().next()).await {
 				match message {
 					Some(Ok(tungstenite::Message::Binary(message_msgpack))) => {
 						*self.last_received_at.lock().unwrap() = Instant::now();
